﻿using Hotline.Caches;
using Hotline.CallCenter.Calls;
using Hotline.CallCenter.Devices;
using Hotline.CallCenter.Ivrs;
using Hotline.Settings;
using Hotline.Share.Enums;
using Hotline.Share.Notifications;
using MediatR;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using NewRock.Sdk;
using NewRock.Sdk.Transfer.Connect.Request;
using XF.Domain.Cache;
using XF.Domain.Constants;

namespace Hotline.Application.Handlers.CallCenter.FlowControl
{
    public class IncomingNotificationHandler : INotificationHandler<IncomingNotification>
    {
        private readonly ICallRepository _callRepository;
        private readonly ICallDetailRepository _callDetailRepository;
        private readonly ISystemSettingCacheManager _systemSettingCacheManager;
        //private readonly ITelCacheManager _telCacheManager;
        private readonly IIvrCacheManager _ivrCacheManager;
        private readonly INewRockClient _newRockClient;
        private readonly IOptionsSnapshot<DeviceConfigs> _options;
        private readonly ITypedCache<WorkTimeSettings> _worktimeCache;
        private readonly IOptionsSnapshot<WorkTimeSettings> _worktimeOptions;
        private readonly ILogger<IncomingNotificationHandler> _logger;

        public IncomingNotificationHandler(
            ICallRepository callRepository, ICallDetailRepository callDetailRepository,
            ISystemSettingCacheManager systemSettingCacheManager, IIvrCacheManager ivrCacheManager,
            INewRockClient newRockClient, IOptionsSnapshot<DeviceConfigs> options,
            ITypedCache<WorkTimeSettings> worktimeCache, IOptionsSnapshot<WorkTimeSettings> worktimeOptions,
            ILogger<IncomingNotificationHandler> logger)
        {
            _callRepository = callRepository;
            _callDetailRepository = callDetailRepository;
            _systemSettingCacheManager = systemSettingCacheManager;
            _ivrCacheManager = ivrCacheManager;
            _newRockClient = newRockClient;
            _options = options;
            _worktimeCache = worktimeCache;
            _worktimeOptions = worktimeOptions;
            _logger = logger;
        }

        public async Task Handle(IncomingNotification notification, CancellationToken cancellationToken)
        {
            var model = await _callRepository.GetAsync(
                x => x.ConversationId == notification.Visitor.Id && x.Trunk == notification.TrunkId &&
                     x.FromNo == notification.Visitor.From && x.CreationTime >= DateTime.Now.AddHours(-2), cancellationToken);
            if (model != null)
            {
                model.CallStatus = ECallStatus.Incoming;
                await _callRepository.UpdateAsync(model, cancellationToken);
                var detail = new CallDetail()
                {
                    CallId = model.Id,
                    CallStatus = ECallStatus.Incoming,
                    OMCallId = notification.Visitor.CallId,
                    ConversationId = notification.Visitor.Id,
                    EventName = notification.Attribute,
                    FromNo = notification.Visitor.From,
                    ToNo = notification.Visitor.To,
                };
                await _callDetailRepository.AddAsync(detail, cancellationToken);
            }
            //TODO IVR处理
            var setting = _systemSettingCacheManager.GetSetting(SettingConstants.IVRConfig);
            if (bool.Parse(setting.SettingValue))
            {
                //TODO 获取工作或休息时间（接听策略）
                //var ivrList = _ivrCacheManager.GetIvrs();
                //var ivr = ivrList.First(x => x.IvrCategoryId == "08da9b9f-a35d-4ade-8ea7-55e8abbcdefd" && x.IsRoot);

                var ivr = GetCorrectIvr();
                _logger.LogInformation("transfer to ivr.no: {ivrNo}", ivr.No);
                await _newRockClient.VisitorToMenu(
                    new VisitorToMenuRequest()
                    {
                        Attribute = "Connect",
                        Menu = new VisitorToMenuMenu() { Id = ivr.No },
                        Visitor = new VisitorToMenuVisitor() { Id = notification.Visitor.Id }
                    },
                    _options.Value.ReceiveKey, _options.Value.Expired, cancellationToken);
            }
            else
            {
                //TODO 跳转默认分机组
            }
        }

        private Ivr GetCorrectIvr()
        {
            var worktimeSettings = _worktimeCache.GetOrAdd("worktimesettings", d => _worktimeOptions.Value, ExpireMode.Absolute, TimeSpan.FromDays(1));
            var categoryId = GetCorrectCategory(worktimeSettings);
            var ivrList = _ivrCacheManager.GetIvrs();
            var ivr = ivrList.First(x => x.IvrCategoryId == categoryId && x.IsRoot);
            return ivr;
        }

        private string GetCorrectCategory(WorkTimeSettings settings)
        {
            if (!settings.WorkDay.Contains((int)DateTime.Now.DayOfWeek))
                return settings.RestCategory;
            var time = TimeOnly.FromDateTime(DateTime.Now);
            if ((time >= TimeOnly.Parse(settings.MorningBegin) && time <= TimeOnly.Parse(settings.MorningEnd))
                || (time >= TimeOnly.Parse(settings.AfterBegin) && time <= TimeOnly.Parse(settings.AfterEnd))
                )
                return settings.WorkCategory;
            return settings.RestCategory;
        }
    }
}
