﻿using System.Linq.Expressions;
using SqlSugar;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Entities;

namespace Hotline.Repository.SqlSugar
{
    public static class QueryExtensions
    {
        public static ISugarQueryable<TEntity> DataPermissionFiltering<TEntity>(this ISugarQueryable<TEntity> queryable, IDataPermissionFilterBuilder dataPermissionFilterBuilder)
            where TEntity : class, IEntity<string>, IDataPermission, new()
        {
            return queryable.Where(dataPermissionFilterBuilder.Build<TEntity>());
        }
    }

    public interface IDataPermissionFilterBuilder
    {
        Expression<Func<TEntity, bool>> Build<TEntity>()
            where TEntity : class, IEntity<string>, IDataPermission, new();
    }

    public class DataPermissionFilterBuilder : IDataPermissionFilterBuilder, IScopeDependency
    {
        private readonly ISessionContext _sessionContext;
        private readonly IDataPermissionManager _dataPermissionManager;

        public DataPermissionFilterBuilder(ISessionContext sessionContext, IDataPermissionManager dataPermissionManager)
        {
            _sessionContext = sessionContext;
            _dataPermissionManager = dataPermissionManager;
        }

        public Expression<Func<TEntity, bool>> Build<TEntity>() where TEntity : class, IEntity<string>, IDataPermission, new()
        {
            var userId = _sessionContext.RequiredUserId;
            var scheme = _dataPermissionManager.GetQueryFilter<TEntity>(userId);
            switch (scheme.QueryFilter)
            {
                case EQueryFilter.CreatorOnly:
                    return d => d.CreateUserId == userId;
                case EQueryFilter.SelfAndSubDepartment:
                    return d => d.OrgCode == scheme.OrgCode;//todo
                default:
                    throw new ArgumentOutOfRangeException();
            }
        }
    }

    public interface IDataPermissionManager
    {
        DataPermissionScheme GetQueryFilter<TEntity>(string userId) where TEntity : class, IEntity<string>, IDataPermission, new();
    }

    public record DataPermissionScheme(EQueryFilter QueryFilter, string? OrgCode = "", string? AreaId = "");

    public class DataPermissionManager : IDataPermissionManager
    {
        public DataPermissionScheme GetQueryFilter<TEntity>(string userId) where TEntity : class, IEntity<string>, IDataPermission, new()
        {
            EQueryFilter? queryFilter = GetCurrentQueryFilter(userId, typeof(TEntity).Name);
            if (queryFilter == null)
            {
                //todo 未配置数据查询权限处理方案
            }

            if (queryFilter == EQueryFilter.CreatorOnly)
                return new DataPermissionScheme(EQueryFilter.CreatorOnly);

            if (queryFilter == EQueryFilter.SelfAndSubDepartment)
            {
                //todo 查询对应部门编码

                return new DataPermissionScheme(EQueryFilter.SelfAndSubDepartment, "");
            }

            throw new NotImplementedException();
        }

        private EQueryFilter? GetCurrentQueryFilter(string userId, string entityName)
        {
            throw new NotImplementedException();
        }
    }
}
